<?php

declare(strict_types=1);

namespace Dedi\SyliusMailAdminPlugin\Handler;

use Dedi\SyliusMailAdminPlugin\Event\MailPreRenderEvent;
use Dedi\SyliusMailAdminPlugin\Factory\TrackingUrlFactoryInterface;
use Sylius\Component\Core\Model\OrderInterface;
use Sylius\Component\Core\Model\ShipmentInterface;
use Sylius\Component\Core\Model\ShippingMethodInterface;
use Webmozart\Assert\Assert;

class ShipmentConfirmationHandler implements MailVariablesHandlerInterface
{
    private TrackingUrlFactoryInterface $trackingUrlFactory;

    public function __construct(TrackingUrlFactoryInterface $trackingUrlFactory)
    {
        $this->trackingUrlFactory = $trackingUrlFactory;
    }

    public function handle(MailPreRenderEvent $event): MailPreRenderEvent
    {
        $data = $event->getData();

        /** @var OrderInterface $order */
        $order = $data['order'];
        Assert::isInstanceOf($order, OrderInterface::class);

        /** @var ShipmentInterface $shipment */
        $shipment = $order->getShipments()->first();
        Assert::isInstanceOf($shipment, ShipmentInterface::class);

        /** @var ShippingMethodInterface $method */
        $method = $shipment->getMethod();

        $data['tracking_url'] = $this->trackingUrlFactory->build($method, $shipment->getTracking());

        $event->setData($data);

        return $event;
    }
}
